/*
	Copyright (C) 2004 - 2005 rt2x00 SourceForge Project
	<http://rt2x00.serialmonkey.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the
	Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
	Module Name: rt2x00core.h

	Abstract: Data structures for the rt2x00 core module.
	Supported chipsets: RT2460, RT2560 & RT2570.

	Revision History:
	Who		When		What
	--------	-----------	-----------------------------
	IvD		3 Sep 2005	Initial Release (Beta 1).
 */ 

#ifndef RT2X00CORE_H
#define RT2X00CORE_H

#include "ieee80211/net/ieee80211.h"

/*
 * NOTE: Lock ordering.
 * Locks should be acquired in the following order:
 *	device->rt2x00_sem
 *	ieee80211->lock
 *	ring->lock
 * Locks should be released in reversed order.
 */

/* 
 * Private IOCTL commands
 */
#define SIOCSIWPRIVNETWORK		SIOCIWFIRSTPRIV + 0x0000
#define SIOCGIWPRIVNETWORK		SIOCIWFIRSTPRIV + 0x0001
#define SIOCSIWPRIVPREAMBLE		SIOCIWFIRSTPRIV + 0x0002
#define SIOCGIWPRIVPREAMBLE		SIOCIWFIRSTPRIV + 0x0003
#define SIOCSIWPRIVANTENNA		SIOCIWFIRSTPRIV + 0x0004
#define SIOCGIWPRIVANTENNA		SIOCIWFIRSTPRIV + 0x0005
#define SIOCSIWPRIVGEOGRAPHY		SIOCIWFIRSTPRIV + 0x0006
#define SIOCGIWPRIVGEOGRAPHY		SIOCIWFIRSTPRIV + 0x0007

/*
 * Private IOCTL flags.
 */
#define IOCTL_PREAMBLE_LONG		0x01	/* SIOCSIWPRIVPREAMBLE */
#define IOCTL_PREAMBLE_SHORT		0x02	/* SIOCSIWPRIVPREAMBLE */
#define IOCTL_ANTENNA_TX_DIV		0x01	/* SIOCSIWPRIVANTENNA */
#define IOCTL_ANTENNA_TX_A		0x02	/* SIOCSIWPRIVANTENNA */
#define IOCTL_ANTENNA_TX_B		0x04	/* SIOCSIWPRIVANTENNA */
#define IOCTL_ANTENNA_RX_DIV		0x10	/* SIOCSIWPRIVANTENNA */
#define IOCTL_ANTENNA_RX_A		0x20	/* SIOCSIWPRIVANTENNA */
#define IOCTL_ANTENNA_RX_B		0x40	/* SIOCSIWPRIVANTENNA */
#define IOCTL_GEOGRAPHY_USA		0x00	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_CANADA		0x01	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_EUROPE		0x02	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_SPAIN		0x03	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_FRANCE		0x04	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_JAPAN		0x05	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_JAPAN1		0x06	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_ISRAEL		0x07	/* SIOCSIWPRIVGEOGRAPHY */
#define IOCTL_GEOGRAPHY_INVAL		0x08	/* SIOCSIWPRIVGEOGRAPHY */

/*
 * Geography data.
 */
static const struct ieee80211_geo geography_data[IOCTL_GEOGRAPHY_INVAL + 1] = {
	[IOCTL_GEOGRAPHY_USA] = {
		"US",
		.bg_channels = 11,
		.bg = {
			{ 2412, 1}, { 2417, 2}, { 2422, 3}, { 2427, 4},
			{ 2432, 5}, { 2437, 6}, { 2442, 7}, { 2447, 8},
			{ 2452, 9}, { 2457, 10}, { 2462, 11}
		},
		.a_channels = 12,
		.a = {
			{ 5180, 36}, { 5200, 40}, { 5220, 44}, { 5240, 48},
			{ 5260, 52}, { 5280, 56}, { 5300, 60}, { 5320, 64},
			{ 5745,149}, { 5765,153}, { 5785,157}, { 5805,161},
		},
	},
	[IOCTL_GEOGRAPHY_CANADA] = {
		"CA",
		.bg_channels = 11,
		.bg = {
			{ 2412, 1}, { 2417, 2}, { 2422, 3}, { 2427, 4},
			{ 2432, 5}, { 2437, 6}, { 2442, 7}, { 2447, 8},
			{ 2452, 9}, { 2457, 10}, { 2462, 11}
		},
		.a_channels = 12,
		.a = {
			{ 5180, 36}, { 5200, 40}, { 5220, 44}, { 5240, 48},
			{ 5260, 52}, { 5280, 56}, { 5300, 60}, { 5320, 64},
			{ 5745,149}, { 5765,153}, { 5785,157}, { 5805,161},
		},
	},
	[IOCTL_GEOGRAPHY_EUROPE] = {
		"EU",
		.bg_channels = 13,
		.bg = {
			{ 2412, 1}, { 2417, 2}, { 2422, 3}, { 2427, 4},
			{ 2432, 5}, { 2437, 6}, { 2442, 7}, { 2447, 8},
			{ 2452, 9}, { 2457, 10}, { 2462, 11}, { 2467, 12},
			{ 2472, 13}
		},
		.a_channels = 11,
		.a = {
			{ 5500, 100 }, { 5520, 104 }, { 5540, 108 }, { 5560, 112 },
			{ 5580, 116 }, { 5600, 120 }, { 5620, 124 }, { 5640, 128 },
			{ 5660, 132 }, { 5680, 136 }, { 5700, 140},
		},
	},
	[IOCTL_GEOGRAPHY_SPAIN] = {
		"ES",
		.bg_channels = 2,
		.bg = {
			{ 2457, 10}, { 2462, 11}
		},
		.a_channels = 11,
		.a = {
			{ 5500, 100 }, { 5520, 104 }, { 5540, 108 }, { 5560, 112 },
			{ 5580, 116 }, { 5600, 120 }, { 5620, 124 }, { 5640, 128 },
			{ 5660, 132 }, { 5680, 136 }, { 5700, 140},
		},
	},
	[IOCTL_GEOGRAPHY_FRANCE] = {
		"FR",
		.bg_channels = 4,
		.bg = {
			{ 2457, 10}, { 2462, 11}, { 2467, 12}, { 2472, 13}
		},
		.a_channels = 11,
		.a = {
			{ 5500, 100 }, { 5520, 104 }, { 5540, 108 }, { 5560, 112 },
			{ 5580, 116 }, { 5600, 120 }, { 5620, 124 }, { 5640, 128 },
			{ 5660, 132 }, { 5680, 136 }, { 5700, 140},
		},
	},
	[IOCTL_GEOGRAPHY_JAPAN] = {
		"JP",
		.bg_channels = 3,
		.bg = {
			{ 2457, 10}, { 2462, 11}, { 2484, 14}
		},
		.a_channels = 0,
	},
	[IOCTL_GEOGRAPHY_JAPAN1] = {
		"JP1",
		.bg_channels = 13,
		.bg = {
			{ 2412, 1}, { 2417, 2}, { 2422, 3}, { 2427, 4},
			{ 2432, 5}, { 2437, 6}, { 2442, 7}, { 2447, 8},
			{ 2452, 9}, { 2457, 10}, { 2462, 11}, { 2467, 12},
			{ 2472, 13}
		},
		.a_channels = 0,
	},
	[IOCTL_GEOGRAPHY_ISRAEL] = {
		"IL",
		.bg_channels = 7,
		.bg = {
			{ 2422, 3}, { 2427, 4}, { 2432, 5}, { 2437, 6},
			{ 2442, 7}, { 2447, 8}, { 2452, 9}
		},
		.a_channels = 0,
	},
	[IOCTL_GEOGRAPHY_INVAL] = {
		"??",
		.bg_channels = 2,
		.bg = {
			{ 2457, 10}, { 2462, 11}
		},
		.a_channels = 0,
	},
};

/*
 * Core data structures.
 */
struct _rt2x00_connect{
	/*
	 * Time of last status update.
	 * When jiffies > (timestamp + CONNECT_TIMEOUT)
	 * Connection attempt has failed.
	 */
	unsigned long			timestamp;
#define CONNECT_TIMEOUT			( HZ / 2 )

	/*
	 * Bssid we are trying to connect to.
	 */
	u8				bssid[ETH_ALEN];

	/*
	 * Current connection status.
	 */
	u16				status;
#define CONNECT_FAIL			0x0000
#define CONNECT_AUTHENTICATE		0x0001
#define CONNECT_HANDSHAKE		0x0002
#define CONNECT_ASSOCIATE		0x0003
#define CONNECT_SUCCESS			0x0004
} __attribute__ ((packed));

struct _packet_ring{
	spinlock_t			lock;
	struct _rt2x00_core		*core;
	struct work_struct		work;
	struct sk_buff_head		queue;
	u8				ready;
	u8				type;
	u16				__pad;		/* For alignment only. */
} __attribute__ ((packed));

struct _rt2x00_core{
	struct _rt2x00_device		*device;

	/*
	 * Ieee80211 stack structure.
	 */
	struct ieee80211_device		*ieee80211;

	/*
	 * Device configuration.
	 */
	struct _rt2x00_config		config;

	/*
	 * Queue for pending Scan and link tuning work.
	 */
	struct workqueue_struct		*workqueue;

	/*
	 * Work structure for periodic channel switching during scan period.
	 */
	struct work_struct		scan_tuner;
	u16				scan_flags;
	u16				_pad;		/* For alignment only. */

	/*
	 * Work structure for periodic link handling.
	 */
	struct work_struct		link_tuner;

	/*
	 * Network we are connected to.
	 */
	struct ieee80211_network	*network;

	/*
	 * Authentication/Association details.
	 */
	struct _rt2x00_connect		connect;

	/*
	 * Ring name: RX.
	 * Description: Packet ring used for receiving packets.
	 */
	struct _packet_ring		rx;

	/*
	 * Ring name: TX.
	 * Description: Packet ring used for transmitting normal packets.
	 */
	struct _packet_ring		tx;

	/*
	 * Ring name: ATIM.
	 * Description: Packet ring used for transmitting power management packets.
	 */
	struct _packet_ring		atim;

	/*
	 * Ring name: PRIO.
	 * Description: Packet ring used for transmitting management packets.
	 */
	struct _packet_ring		prio;

	/*
	 * Ring name: BEACON.
	 * Description: Packet ring used for transmitting beacon packets.
	 */
	struct _packet_ring		beacon;

	/*
	 * Network statistics.
	 */
	struct iw_statistics		wireless_stats;
	struct iw_public_data 		wireless_data;
} __attribute__ ((packed));

static inline int
rt2x00_is_cck_bitrate(const u8 bitrate)
{
	return (bitrate == IEEE80211_CCK_RATE_1MB ||
		bitrate == IEEE80211_CCK_RATE_2MB ||
		bitrate == IEEE80211_CCK_RATE_5MB ||
		bitrate == IEEE80211_CCK_RATE_11MB);
}

static inline int
rt2x00_is_ofdm_bitrate(const u8 bitrate)
{
	return (bitrate == IEEE80211_OFDM_RATE_6MB ||
		bitrate == IEEE80211_OFDM_RATE_9MB ||
		bitrate == IEEE80211_OFDM_RATE_12MB ||
		bitrate == IEEE80211_OFDM_RATE_18MB ||
		bitrate == IEEE80211_OFDM_RATE_24MB ||
		bitrate == IEEE80211_OFDM_RATE_36MB ||
		bitrate == IEEE80211_OFDM_RATE_48MB ||
		bitrate == IEEE80211_OFDM_RATE_54MB);
}

static inline int
rt2x00_is_24ghz_channel(const u8 channel)
{
	return (channel >= IEEE80211_24GHZ_MIN_CHANNEL && channel <= IEEE80211_24GHZ_MAX_CHANNEL);
}

static inline int
rt2x00_is_52ghz_channel(const u8 channel)
{
	return (channel >= IEEE80211_52GHZ_MIN_CHANNEL && channel <= IEEE80211_52GHZ_MAX_CHANNEL);
}

static inline u16
rt2x00_get_sequence(struct _rt2x00_device *device)
{
	struct _rt2x00_core		*core = rt2x00_core(device);
	u16				sequence = 0x0000;

	spin_lock(&core->ieee80211->lock);

	sequence = core->config.sequence;
	core->config.sequence = (core->config.sequence + 0x0010) & IEEE80211_SCTL_SEQ;

	spin_unlock(&core->ieee80211->lock);

	return sequence;
}

static inline u16
rt2x00_channel_to_freq(u8 channel)
{
	if(channel <= 14)
		return ((channel == 14) ? 2484 : (channel * 5) + 2407);
	else
		return ((channel + 1000) * 5);
}

#endif /* RT2X00CORE_H */
