#!/usr/bin/env python

import os, sys

# ----------------------------------------------------------------------------

def get_params( num_of_args ) :
    """Get wireless essid, wep key etc from command line arguments"""

    Usage = '%s card_type essid [ wep_key ]\n' % sys.argv[ 0 ]
    Usage = '%s\twhere card_type is rt61 or rt73' % Usage

    if ( num_of_args < 3 ) or ( num_of_args > 4 ) :
        print Usage
        sys.exit()

    card = sys.argv[ 1 ].lower()
    if card not in ( 'rt61', 'rt73', ) :
        print Usage
        sys.exit()

    essid = sys.argv[ 2 ]

    if num_of_args == 3 :
        sys.argv.append( 'off' )

    if sys.argv[ 3 ] == 'off' :
        key_type = None
    else :
        key_type = 'WEP'

    cryptkey = sys.argv[ 3 ]

    return card, essid, key_type, cryptkey

# ----------------------------------------------------------------------------

def get_user_input() :
    """Get wireless essid, wep key etc from user input"""

    variant = os.getenv( 'variant' )
    if variant == '61' :
        card = 'rt61'
    elif variant == '73' :
        card = 'rt73'
        variant = None
    else :
        prompt = """\nPlease choose Ralink card type you're setting up:

\t1. RT61 - PCI and PCMCIA
\t2. RT73 - USB dongle

\tPlease choose 1 or 2 (q quits) >> """
        while True :
            sys.stdout.write( prompt )
            reply = sys.stdin.readline().strip().upper()
            if reply == 'Q' :
                sys.exit()
            elif reply == '1' :
                card = 'rt61'
                break
            elif reply == '2' :
                card = 'rt73'
                break
            else :
                sys.stdout.write( '\nPlease answer 1, 2 or q\n' )

    prompt = '\nPlease enter your essid (network id) >> '
    sys.stdout.write( prompt )
    essid = sys.stdin.readline().strip()
    if not essid :
        sys.exit()

    prompt = """\nPlease choose encryption type:

\t1. Open - data is not encrypted (insecure!)
\t2. WEP encryption

\tPlease choose 1 or 2 (q quits) >> """
    while True :
        sys.stdout.write( prompt )
        reply = sys.stdin.readline().strip().upper()
        if reply == 'Q' :
            sys.exit()
        elif reply == '1' :
            key_type = None
            cryptkey = 'off'
            break
        elif reply == '2' :
            key_type = 'WEP'
            break
        else :
            sys.stdout.write( '\nPlease answer 1, 2 or q\n' )

    if key_type == 'WEP' :
        valid_chars = '01234567890abcdef'
        prompt = """\nYou have chosen WEP encryption. This needs a hexadecimal key,
10 characters for 64 bit or 26 for 128 bit encryption.\n"""
        sys.stdout.write( prompt )
        prompt = """\nPlease enter your hexadecimal encryption key >> """
        while True :
            sys.stdout.write( prompt )
            cryptkey = sys.stdin.readline().strip().lower()
            if cryptkey == 'q' or not cryptkey :
                sys.exit()
            if len( cryptkey ) not in ( 10, 26 ) :
                sys.stdout.write( 'I need 10 or 26 hex characters\n' )
                continue
            hex = True
            for char in cryptkey :
                if char not in valid_chars :
                    hex = False
            if not hex :
                sys.stdout.write( 'I need all hex characters\n' )
            else :
                break
                
    return card, essid, key_type, cryptkey

# ----------------------------------------------------------------------------

def write_suse_script( card, essid, key_type, cryptkey, ) :
    """create a script to start up wifi on SUSE 10.2"""

    if not key_type :
         key = 'off'
    else :
         key = cryptkey

    script="""#!/bin/sh

# start_wifi: temp script to start Ralink wifi on SUSE 10.2

# these variables are set during system configuration
essid=%s
key=%s
driver=%s

if [ $driver = rt61 ]
  then
    iface=ra0
  else
    iface=rausb0
fi

dhclient -r $iface
ifdown $iface
modprobe -r $driver

if [ x$1 = xreset ]
  then
    /usr/local/bin/set_essid.py $driver $essid $key
    sync;sync
    sleep 2
fi
modprobe $driver
sleep 1
if [ $driver = rt61 ]
  then
    # not necessary for rt73 USB driver
    iwconfig $iface essid $essid key $key
    sleep 3
fi
dhclient $iface
""" % ( essid, key, card, )

    try :
        f = open( 'start_wifi', 'w' )
    except IOError :
        print 'Unable to open start_wifi for writing'
        raise
    f.write( script )
    f.close()

    return

# ----------------------------------------------------------------------------

L = len( sys.argv )

if L == 1 :
    card, essid, key_type, cryptkey = get_user_input()
else :
    card, essid, key_type, cryptkey = get_params( L )
write_suse_script( card, essid, key_type, cryptkey, )

Testing = False
if Testing :
    sys.exit()

fw_dir = '/etc/Wireless/%sSTA' % card.upper()
conf = '%s/%ssta.dat' % ( fw_dir, card )

# open Ralink config file
try :
    cf = open( conf, 'r' )
except IOError :
    print "Unable to open %s config file (%s) for read" % ( card, conf )
    raise
data = cf.read()
cf.close()

lines = data.split( '\n' )

for i in range( len(lines) ) :
    if lines[ i ].find( 'SSID=' ) >=0 :
        lines[ i ] = 'SSID=%s' % essid
    if lines[ i ].find( 'EncrypType=' ) >=0 :
        if key_type :
            lines[ i ] = 'EncrypType=%s' % key_type
        else :
            lines[ i ] = '#EncrypType=NONE'
    if lines[ i ].find( 'DefaultKeyID=' ) >=0 :
        if key_type :
            lines[ i ] = 'DefaultKeyID=1'
        else :
            lines[ i ] = '#DefaultKeyID=1'
    if lines[ i ].find( 'Key1Str=' ) >=0 :
        if key_type :
            lines[ i ] = 'Key1Str=%s' % cryptkey
        else :
            lines[ i ] = '#Key1Str='
    if lines[ i ].find( 'AuthMode=' ) >=0 :
        if key_type :
            lines[ i ] = 'AuthMode=SHARED'
        else :
            lines[ i ] = '#AuthMode=SHARED'
    if lines[ i ].find( 'Key1Type=' ) >=0 :
        if key_type :
            lines[ i ] = 'Key1Type=0'
        else :
            lines[ i ] = '#Key1Type=0'

data = '\n'.join( lines )

try :
    cf = open( conf, 'w' )
except IOError :
    print "Unable to open %s config file (%s) for write" % ( card, conf )
    raise

cf.write( data )
cf.close()
